// SoundListMap.cpp: implementation of the SoundListMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "pop.h"
#include "SoundListMap.h"

#include "dsutil.h"
#include "dxutil.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// SoundList:
// keeps a list of a specific sound
//
CSoundList::CSoundList()
{

}

CSoundList::~CSoundList()
{
	RemoveSounds(TRUE);
	ASSERT(IsEmpty());
}

void CSoundList::RemoveSounds(BOOL removePlaying)
{
	CSound * sound = NULL;
	while( !IsEmpty() )
	{
		sound = GetHead();
		RemoveHead();
		SAFE_DELETE( sound );
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSoundListMap:
// keeps a map of sounds we've played so far
//
CSoundListMap::CSoundListMap()
{

}

CSoundListMap::~CSoundListMap()
{
	RemoveSounds(TRUE);
	CString name;
	CSoundList * soundList = NULL;
	POSITION pos = GetStartPosition();
	while( pos )
	{
		GetNextAssoc(pos,name,soundList);
		SAFE_DELETE( soundList );
	}
}

CSound* CSoundListMap::AddBuffer(LPCTSTR soundName, CSound* sound)
{
	CSoundList * soundList = NULL;
	if ( !Lookup(soundName,soundList) )
	{
		soundList = new CSoundList();
		SetAt(soundName,soundList);
	}

	ASSERT( soundList );
	soundList->AddTail(sound);
	return sound;
}

CSound* CSoundListMap::FindBuffer(LPCTSTR soundName)
{
	CSoundList * soundList = NULL;
	CSound * sound = NULL;
	if ( Lookup(soundName,soundList) )
	{ // we found the list, now see if we can find a free buffer
		POSITION pos = soundList->GetHeadPosition();
		while(pos)
		{
			sound = soundList->GetNext(pos);
			if ( !sound->IsSoundPlaying() )
			{
				return sound;
			}
		}
	}
	return NULL;
}

// free all buffers
void CSoundListMap::RemoveSounds(BOOL removePlaying)
{
	if( IsEmpty() )
		return;

	// delete any sounds we might have happened to load
	POSITION pos = GetStartPosition();
	CString name;
	CSoundList * soundList = NULL;
	while(pos)
	{
		GetNextAssoc(pos,name,soundList);
		if( soundList )
		{
			soundList->RemoveSounds(removePlaying);
		}
	}
}

// free buffers of specified sound
void CSoundListMap::RemoveSounds(LPCTSTR soundName, BOOL removePlaying)
{
	CSoundList * soundList = NULL;
	Lookup(soundName,soundList);
	if(soundList)
		soundList->RemoveSounds(removePlaying);
}